include <ctype.h>
include <error.h>

define STDFEEDS  "wyffos$lib/stdfeeds"
define STDCROSS  "wyffos$lib/cross"
define STDSIZE   24.0
define MAXFIBS   160
define SZFTYPE   2
define SZNAME    18
define SZDATA    64
define SZCOOR    11
define SZMAG     9

procedure t_headid()

# Local variables

pointer sp,extn,object,apref,cross,shifts,apidtable,feed,feed2,obstype,fldnm
pointer optr,aptr
char obj[SZ_FNAME],apr[SZ_FNAME],msg[SZ_LINE],keyword[8],comment[SZ_LINE]
char name[SZNAME],ra[SZCOOR],dec[SZCOOR],headinfo[SZDATA],mag[SZMAG]
char ftype[SZFTYPE]
real pixsize,width,sep,stdshift,minsep,maxsep,lower,upper,shiftpix
bool match,refapid,skip
int fd,nfibs,i,j,icross[MAXFIBS],nspec,nskip,beamtype

# Functions

pointer immap()
real imgetr()
int open(),fscan(),strlen(),access()
bool streq()

begin

      # Get workspace

      call smark(sp)
      call salloc(extn,SZ_FNAME,TY_CHAR)
      call salloc(object,SZ_FNAME,TY_CHAR)
      call salloc(apref,SZ_FNAME,TY_CHAR)
      call salloc(cross,SZ_FNAME,TY_CHAR)
      call salloc(shifts,SZ_FNAME,TY_CHAR)
      call salloc(apidtable,SZ_FNAME,TY_CHAR)
      call salloc(feed,SZ_LINE,TY_CHAR)
      call salloc(feed2,SZ_LINE,TY_CHAR)
      call salloc(obstype,SZ_LINE,TY_CHAR)
      call salloc(fldnm,SZ_LINE,TY_CHAR)

      # Get the file extension

      call envgets("imtype",Memc[extn],SZ_FNAME)

      # Get the name of the object file and the aperture reference file
      # Try and open them. Get the name of the feed from the apref.

      call clgstr("object",obj,SZ_FNAME)
      call strpextn(obj,Memc[extn],Memc[object],SZ_FNAME)
      iferr (optr = immap(Memc[object],READ_ONLY,0)) {
         call eprintf("Can't open: %s\n")
            call pargstr(Memc[object])
         call erract(EA_ERROR)
      }
      call clgstr("apref",apr,SZ_FNAME)
      call strpextn(apr,Memc[extn],Memc[apref],SZ_FNAME)
      iferr (aptr = immap(Memc[apref],READ_ONLY,0)) {
         call eprintf("Can't open: %s\n")
            call pargstr(Memc[apref])
         call erract(EA_ERROR)
      }
      call imgstr(aptr,"WYFFEED",Memc[feed],SZ_LINE)
      pixsize = imgetr(aptr,"CCDYPIXE")
      if (pixsize == 0.0)
         pixsize = STDSIZE
      else if (pixsize < 1.0)
         pixsize = pixsize * 1.0e6

      # Read the standard feed file and match up the input feed.
      # If you find a match, then read out the number of fibres and the
      # spacing between each.

      match = false
      fd = open(STDFEEDS,READ_ONLY,TEXT_FILE)
      while (fscan(fd) != EOF) {
         call gargwrd(Memc[feed2],SZ_LINE)
         if (streq(Memc[feed2],Memc[feed])) {
            match = true
            call gargi(nfibs)
            call gargr(width)
            call gargr(sep)
            call gargr(stdshift)
            sep = sep/pixsize
            width = width/pixsize
            stdshift = stdshift/pixsize
            minsep = sep - 1.0
            maxsep = sep + 2.0
            lower = -real(nint(width*0.5))
            upper = -lower
            break
         }
      }
      call close(fd)

      # If no match was found then signal an error.

      if (!match) {
         call sprintf(msg,SZ_LINE,"Non-standard feed name: %s\n")
            call pargstr(Memc[feed])
         call imunmap(optr)
         call imunmap(aptr)
         call sfree(sp)
         call error(0,msg)
      }

      # Now open up the spectrum vs. pivot cross reference file and
      # read it

      call strcpy(STDCROSS,Memc[cross],SZ_FNAME)
      call strcat(Memc[feed],Memc[cross],SZ_FNAME)
      fd = open(Memc[cross],READ_ONLY,TEXT_FILE)
      nspec = 0
      i = fscan(fd)
      while (fscan(fd) != EOF) {
         call gargi(i)
         call gargi(j)
         icross[i] = j
         nspec = nspec + 1
      }
      call close(fd)

      # Now open a shifts file for this feed

      call sprintf(Memc[shifts],SZ_FNAME,"sh%s")
         call pargstr(Memc[apref])
      if (access(Memc[shifts],0,0) == YES)
         call delete(Memc[shifts])
      fd = open(Memc[shifts],NEW_FILE,TEXT_FILE)
      do i = 1,3 {
         shiftpix = -real(i-1)*stdshift
         call fprintf(fd,"%0.6g %dx3\n")
            call pargr(shiftpix)
            call pargi(i)
      }
      call close(fd)

      # What sort of file is the aperture reference?

      call imgstr(aptr,"OBSTYPE",Memc[obstype],SZ_LINE)
      if (streq(Memc[obstype],"SKY") || streq(Memc[obstype],"FLAT"))
         refapid = true
      else
         refapid = false

      # Create a name for the aperture id table from the field
      # name in the object file

      call imgstr(optr,"AF2FLDNM",Memc[fldnm],SZ_LINE)
      for (i = 1; i <= strlen(Memc[fldnm]); i = i + 1) {
         if (IS_WHITE(Memc[fldnm+i-1]))
            Memc[fldnm+i-1] = '_'
      }
      call sprintf(Memc[apidtable],SZ_FNAME,"%sapid")
         call pargstr(Memc[fldnm])

      # Now, if the apidtable doesn't already exist, then create it here

      if (access(Memc[apidtable],0,0) == NO) {
         nskip = 0
         skip = true
         fd = open(Memc[apidtable],NEW_FILE,TEXT_FILE)
         
         # Loop for all of the expected spectra

         do i = 1,nspec {
            call sprintf(keyword,8,"F%03dDAT1")
               call pargi(icross[i])
            call imgstr(optr,keyword,headinfo,SZDATA)
            call strcpy(headinfo[44],ftype,SZFTYPE)
            call strcpy(headinfo,name,SZNAME)
            for (j=SZNAME; IS_WHITE(name[j]) && j >= 1; j = j - 1)
               ;
            name[j+1] = EOS

            # Test for the different fibre types. First look 
            # at unused fibres

            if (streq(ftype,"UN")) {
               if (skip) {
                  if (refapid) {
                     if (streq(name,"DISABLED")) {
                        nskip = nskip + 1
                        next
                     } else {
                        skip = false
                        beamtype = -1
                        call strcpy(name,comment,SZ_LINE)
                     }
                  } else {
                     nskip = nskip + 1
                     next
                  }
               } else {
                  if (refapid) {
                     if (streq(name,"MISSING") || streq(name,"DISABLED"))
                        nskip = nskip + 1
                  } else
                     nskip = nskip + 1
                  beamtype = -1
                  call strcpy(name,comment,SZ_LINE)
               }
            } else if (streq(ftype,"PR") || streq(ftype,"SK")) {
               skip = false
               if (streq(ftype,"PR"))
                  beamtype = 1
               else
                  beamtype = 0
               call strcpy(headinfo[20],ra,SZCOOR)
               call strcpy(headinfo[32],dec,SZCOOR)
               call strcpy(headinfo[47],mag,SZMAG)
               call sprintf(comment,SZ_LINE,"%s  %s  %s  %s")
                  call pargstr(name)
                  call pargstr(ra)
                  call pargstr(dec)
                  call pargstr(mag)
            } else {
               nskip = nskip + 1
               next
            }
            call fprintf(fd,"%d %d %s\n")
               call pargi(i)
               call pargi(beamtype)
               call pargstr(comment)
         } 
         nspec = nspec - nskip
         call close(fd)

      # If the aperture id table already exists, then just count the
      # number of spectra...

      } else {
         fd = open(Memc[apidtable],READ_ONLY,TEXT_FILE)
         nspec = 0
         while (fscan(fd) != EOF) {
            call gargi(i)
            call gargi(beamtype)
            call gargstr(comment,SZ_LINE)
            if (!refapid) {
               if (beamtype >= 0)
                  nspec = nspec + 1
            } else {
               call sscan(comment)
                  call gargwrd(name,SZNAME)
               if (beamtype >= 0 || streq(name,"PARKED"))
                  nspec = nspec + 1
            }
         }
         call close(fd)
      }

      # Now send out some info to the user

      call printf("%d %0.6g %0.6g %0.6g %0.6g %0.6g %s %s\n")
         call pargi(nspec)
         call pargr(minsep)
         call pargr(maxsep)
         call pargr(width)
         call pargr(lower)
         call pargr(upper)
         call pargstr(Memc[apidtable])
         call pargstr(Memc[shifts])

      # Close up files and get out of here

      call imunmap(optr)
      call imunmap(aptr)
      call sfree(sp)
end

